@@ -841,24 +841,29 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	GMimeMessage *message = (GMimeMessage *) object;
 	ssize_t nwritten, total = 0;
 	
-	if (!(message->mime_part && g_mime_header_has_raw (message->mime_part->headers))) {
-		/* write the content headers */
-		if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
-			return -1;
-		
-		total += nwritten;
-		
-		if (message->mime_part) {
+	if (message->mime_part) {
+		if (!g_mime_header_has_raw (message->mime_part->headers)) {
+			/* if the mime part has raw headers, then it contains the message
+			 * headers as well otherwise it doesn't, so write them now */
+			if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
+				return -1;
+			
+			total += nwritten;
+			
 			if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
 				return -1;
 			
 			total += nwritten;
 		}
-	}
-	
-	if (message->mime_part) {
-		nwritten = g_mime_object_write_to_stream (message->mime_part, stream);
+		
+		if ((nwritten = g_mime_object_write_to_stream (message->mime_part, stream)) == -1)
+			return -1;
 	} else {
+		if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
+			return -1;
+		
+		total += nwritten;
+		
 		if ((nwritten = g_mime_stream_write (stream, "\n", 1)) == -1)
 			return -1;
 	}
