@@ -2,19 +2,20 @@
 /*  GMime
  *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -41,6 +42,17 @@
 
 #define d(x)
 
+
+/**
+ * SECTION: gmime-part
+ * @title: GMimePart
+ * @short_description: MIME parts
+ * @see_also:
+ *
+ * A #GMimePart represents any MIME leaf part (meaning it has no
+ * sub-parts).
+ **/
+
 /* GObject class methods */
 static void g_mime_part_class_init (GMimePartClass *klass);
 static void g_mime_part_init (GMimePart *mime_part, GMimePartClass *klass);
@@ -429,7 +441,7 @@ mime_part_write_to_stream (GMimeObject *object, GMimeStream *stream)
  * text/plain.
  **/
 GMimePart *
-g_mime_part_new ()
+g_mime_part_new (void)
 {
 	GMimePart *mime_part;
 	GMimeContentType *type;
@@ -900,6 +912,8 @@ g_mime_part_encoding_from_string (const char *encoding)
 		return GMIME_PART_ENCODING_BASE64;
 	else if (!g_ascii_strcasecmp (encoding, "quoted-printable"))
 		return GMIME_PART_ENCODING_QUOTEDPRINTABLE;
+	else if (!g_ascii_strcasecmp (encoding, "uuencode"))
+		return GMIME_PART_ENCODING_UUENCODE;
 	else if (!g_ascii_strcasecmp (encoding, "x-uuencode"))
 		return GMIME_PART_ENCODING_UUENCODE;
 	else
