@@ -2,19 +2,20 @@
 /*  GMime
  *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -28,6 +29,17 @@
 
 #define d(x)
 
+
+/**
+ * SECTION: gmime-stream-cat
+ * @title: GMimeStreamCat
+ * @short_description: A concatenated stream
+ * @see_also: #GMimeStream
+ *
+ * A #GMimeStream which chains together any number of other streams.
+ **/
+
+
 static void g_mime_stream_cat_class_init (GMimeStreamCatClass *klass);
 static void g_mime_stream_cat_init (GMimeStreamCat *stream, GMimeStreamCatClass *klass);
 static void g_mime_stream_cat_finalize (GObject *object);
@@ -415,7 +427,8 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 			}
 			
 			/* after our current position */
-			d(fprintf (stderr, "after cur position in stream[%d] or in a later stream\n"));
+			d(fprintf (stderr, "after cur position in stream[%d] or in a later stream\n",
+				   current->id));
 			do {
 				if (current->stream->bound_end != -1) {
 					len = current->stream->bound_end - current->stream->bound_start;
@@ -434,7 +447,7 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 					break;
 				} else {
 					d(fprintf (stderr, "not within bounds of stream[%d]\n",
-						   current->id, len));
+						   current->id));
 					current->position = len;
 					real += len;
 					
@@ -630,35 +643,24 @@ stream_substream (GMimeStream *stream, off_t start, off_t end)
 		n = n->next;
 	} while (n != NULL);
 	
-	if (s == streams) {
-		d(fprintf (stderr, "returning stream[%d]::substream (%ld, %ld)\n",
-			   n->id, s->start, s->end));
-		substream = g_mime_stream_substream (s->stream, s->start, s->end);
-		while (streams != NULL) {
-			s = streams->next;
-			g_free (streams);
-			streams = s;
-		}
-	} else {
-		d(fprintf (stderr, "returning a substream containing multiple source streams\n"));
-		cat = g_object_new (GMIME_TYPE_STREAM_CAT, NULL);
-		/* Note: we could pass -1 as bound_end, it should Just
-		 * Work(tm) but setting absolute bounds is kinda
-		 * nice... */
-		g_mime_stream_construct (GMIME_STREAM (cat), 0, subend);
-		
-		while (streams != NULL) {
-			s = streams->next;
-			substream = g_mime_stream_substream (streams->stream, streams->start, streams->end);
-			g_mime_stream_cat_add_source (cat, substream);
-			g_object_unref (substream);
-			g_free (streams);
-			streams = s;
-		}
-		
-		substream = (GMimeStream *) cat;
+	d(fprintf (stderr, "returning a substream containing multiple source streams\n"));
+	cat = g_object_new (GMIME_TYPE_STREAM_CAT, NULL);
+	/* Note: we could pass -1 as bound_end, it should Just
+	 * Work(tm) but setting absolute bounds is kinda
+	 * nice... */
+	g_mime_stream_construct (GMIME_STREAM (cat), 0, subend);
+	
+	while (streams != NULL) {
+		s = streams->next;
+		substream = g_mime_stream_substream (streams->stream, streams->start, streams->end);
+		g_mime_stream_cat_add_source (cat, substream);
+		g_object_unref (substream);
+		g_free (streams);
+		streams = s;
 	}
 	
+	substream = (GMimeStream *) cat;
+	
 	return substream;
 	
  error:
