@@ -2,19 +2,20 @@
 /*  GMime
  *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -34,6 +35,18 @@
 int fsync (int fd) { return 0; }
 #endif
 
+
+/**
+ * SECTION: gmime-stream-fs
+ * @title: GMimeStreamFs
+ * @short_description: A low-level FileSystem stream
+ * @see_also: #GMimeStream
+ *
+ * A simple #GMimeStream implementation that sits on top of the
+ * low-level UNIX file descriptor based I/O layer.
+ **/
+
+
 static void g_mime_stream_fs_class_init (GMimeStreamFsClass *klass);
 static void g_mime_stream_fs_init (GMimeStreamFs *stream, GMimeStreamFsClass *klass);
 static void g_mime_stream_fs_finalize (GObject *object);
@@ -67,7 +80,7 @@ g_mime_stream_fs_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeStreamFs),
-			16,   /* n_preallocs */
+			0,    /* n_preallocs */
 			(GInstanceInitFunc) g_mime_stream_fs_init,
 		};
 		
@@ -150,7 +163,8 @@ static ssize_t
 stream_write (GMimeStream *stream, const char *buf, size_t len)
 {
 	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
-	ssize_t nwritten = 0, n;
+	size_t nwritten = 0;
+	ssize_t n;
 	
 	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
 		return -1;
@@ -222,7 +236,6 @@ static int
 stream_reset (GMimeStream *stream)
 {
 	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
-	off_t rv;
 	
 	if (fs->fd == -1)
 		return -1;
@@ -235,10 +248,12 @@ stream_reset (GMimeStream *stream)
 	/* FIXME: if stream_read/write is always going to lseek to
 	 * make sure fd's seek position matches our own, we could just
 	 * set stream->position = stream->bound_start and be done. */
-	if ((rv = lseek (fs->fd, stream->bound_start, SEEK_SET)) != -1)
-		fs->eos = FALSE;
+	if (lseek (fs->fd, stream->bound_start, SEEK_SET) == -1)
+		return -1;
 	
-	return rv;
+	fs->eos = FALSE;
+	
+	return 0;
 }
 
 static off_t
@@ -315,7 +330,7 @@ stream_length (GMimeStream *stream)
 	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
 	off_t bound_end;
 	
-	if (stream->bound_start != -1 && stream->bound_end != -1)
+	if (stream->bound_end != -1)
 		return stream->bound_end - stream->bound_start;
 	
 	bound_end = lseek (fs->fd, 0, SEEK_END);
