@@ -33,6 +33,7 @@
 #else
 #define MAXHOSTNAMELEN 64
 #endif
+#include <sys/utsname.h>
 #include <sys/types.h>
 #include <unistd.h>
 #ifdef HAVE_NETDB_H
@@ -57,6 +58,18 @@
 
 #define d(x)
 
+
+/**
+ * SECTION: gmime-utils
+ * @title: gmime-utils
+ * @short_description: MIME utility functions
+ * @see_also:
+ *
+ * Utility functions to parse, encode and decode various MIME tokens
+ * and encodings.
+ **/
+
+
 #define GMIME_UUENCODE_CHAR(c) ((c) ? (c) + ' ' : '`')
 #define	GMIME_UUDECODE_CHAR(c) (((c) - ' ') & 077)
 
@@ -779,45 +792,80 @@ g_mime_utils_generate_message_id (const char *fqdn)
 #define MUTEX_LOCK()
 #define MUTEX_UNLOCK()
 #endif
-	static unsigned int count = 0;
-	char host[MAXHOSTNAMELEN + 1];
-#ifdef HAVE_GETADDRINFO
-	struct addrinfo hints, *res;
+	static unsigned long int count = 0;
+	const char *hostname = NULL;
 	char *name = NULL;
-#endif
 	char *msgid;
 	
 	if (!fqdn) {
+#ifdef HAVE_UTSNAME_DOMAINNAME
+		struct utsname unam;
+		
+		uname (&unam);
+		
+		hostname = unam.nodename;
+		
+		if (unam.domainname[0])
+			name = g_strdup_printf ("%s.%s", hostname, unam.domainname);
+#else /* ! HAVE_UTSNAME_DOMAINNAME */
+		char host[MAXHOSTNAMELEN + 1];
+		
 #ifdef HAVE_GETHOSTNAME
-		if (gethostname (host, sizeof (host)) == 0) {
-#ifdef HAVE_GETADDRINFO
-			memset (&hints, 0, sizeof (hints));
-			hints.ai_flags = AI_CANONNAME;
+		host[MAXHOSTNAMELEN] = '\0';
+		if (gethostname (host, MAXHOSTNAMELEN) == 0) {
+#ifdef HAVE_GETDOMAINNAME
+			size_t domainlen = MAXHOSTNAMELEN;
+			char *domain;
+			int rv;
 			
-			if (getaddrinfo (host, NULL, &hints, &res) == 0) {
-				name = g_strdup (res->ai_canonname);
-				freeaddrinfo (res);
+			domain = g_malloc (domainlen);
+			
+			while ((rv = getdomainname (domain, domainlen)) == -1 && errno == EINVAL) {
+				domainlen += MAXHOSTNAMELEN;
+				domain = g_realloc (domain, domainlen);
 			}
-#endif /* HAVE_GETADDRINFO */
+			
+			if (rv == 0 && domain[0]) {
+				if (host[0]) {
+					name = g_strdup_printf ("%s.%s", host, domain);
+					g_free (domain);
+				} else {
+					name = domain;
+				}
+			}
+#endif /* HAVE_GETDOMAINNAME */
 		} else {
 			host[0] = '\0';
 		}
-#else
-		host[0] = '\0';
 #endif /* HAVE_GETHOSTNAME */
+		hostname = host;
+#endif /* HAVE_UTSNAME_DOMAINNAME */
 		
 #ifdef HAVE_GETADDRINFO
-		fqdn = name != NULL ? name : (*host ? host : "localhost.localdomain");
-		g_free (name);
-#else
-		fqdn = *host ? host : "localhost.localdomain";
-#endif
+		if (!name && hostname[0]) {
+			/* we weren't able to get a domain name */
+			struct addrinfo hints, *res;
+			
+			memset (&hints, 0, sizeof (hints));
+			hints.ai_flags = AI_CANONNAME;
+			
+			if (getaddrinfo (hostname, NULL, &hints, &res) == 0) {
+				name = g_strdup (res->ai_canonname);
+				freeaddrinfo (res);
+			}
+		}
+#endif /* HAVE_GETADDRINFO */
+		
+		fqdn = name != NULL ? name : (hostname[0] ? hostname : "localhost.localdomain");
 	}
 	
 	MUTEX_LOCK ();
-	msgid = g_strdup_printf ("%ul.%ul.%ul@%s", (unsigned int) time (NULL), getpid (), count++, fqdn);
+	msgid = g_strdup_printf ("%lu.%lu.%lu@%s", (unsigned long int) time (NULL),
+				 (unsigned long int) getpid (), count++, fqdn);
 	MUTEX_UNLOCK ();
 	
+	g_free (name);
+	
 	return msgid;
 }
 
@@ -1401,74 +1449,172 @@ g_mime_utils_best_encoding (const unsigned char *text, size_t len)
 }
 
 
-static char *
-decode_8bit (const char *text, size_t len)
+/**
+ * charset_convert:
+ * @cd: iconv converter
+ * @inbuf: input text buffer to convert
+ * @inleft: length of the input buffer
+ * @outp: pointer to output buffer
+ * @outlenp: pointer to output buffer length
+ * @ninval: the number of invalid bytes in @inbuf
+ *
+ * Converts the input buffer from one charset to another using the
+ * @cd. On completion, @outp will point to the output buffer
+ * containing the converted text (nul-terminated), @outlenp will be
+ * the size of the @outp buffer (note: not the strlen() of @outp) and
+ * @ninval will contain the number of bytes which could not be
+ * converted.
+ *
+ * Bytes which cannot be converted from @inbuf will appear as '?'
+ * characters in the output buffer.
+ *
+ * If *@outp is non-NULL, then it is assumed that it points to a
+ * pre-allocated buffer of length *@outlenp. This is done so that the
+ * same output buffer can be reused multiple times.
+ *
+ * Returns the string length of the output buffer.
+ **/
+static size_t
+charset_convert (iconv_t cd, const char *inbuf, size_t inleft, char **outp, size_t *outlenp, size_t *ninval)
 {
-	const char *fallback_charsets[3] = { "UTF-8", NULL, NULL };
-	size_t inleft, outleft, outlen, rc, min, n;
-	const char **charsets, *best;
-	char *out, *outbuf;
-	const char *inbuf;
-	iconv_t cd;
-	int i;
+	size_t outlen, outleft, rc, n = 0;
+	char *outbuf, *out;
 	
-	if (!(charsets = g_mime_user_charsets ())) {
-		inbuf = g_mime_locale_charset ();
-		if (g_ascii_strcasecmp (inbuf, "UTF-8") != 0)
-			fallback_charsets[1] = inbuf;
-		
-		charsets = fallback_charsets;
+	if (*outp == NULL) {
+		outleft = outlen = (inleft * 2) + 16;
+		outbuf = out = g_malloc (outlen + 1);
+	} else {
+		outleft = outlen = *outlenp;
+		outbuf = out = *outp;
 	}
 	
+	do {
+		rc = iconv (cd, (char **) &inbuf, &inleft, &outbuf, &outleft);
+		if (rc == (size_t) -1) {
+			if (errno == EINVAL) {
+				/* incomplete sequence at the end of the input buffer */
+				n += inleft;
+				break;
+			}
+			
+			if (errno == E2BIG) {
+				/* need to grow the output buffer */
+				outlen += (inleft * 2) + 16;
+				rc = (size_t) (outbuf - out);
+				out = g_realloc (out, outlen + 1);
+				outleft = outlen - rc;
+				outbuf = out + rc;
+			} else {
+				/* invalid byte(-sequence) in the input buffer */
+				*outbuf++ = '?';
+				outleft--;
+				inleft--;
+				inbuf++;
+				n++;
+			}
+		}
+	} while (inleft > 0);
+	
+	iconv (cd, NULL, NULL, &outbuf, &outleft);
+	*outbuf++ = '\0';
+	
+	*outlenp = outlen;
+	*outp = out;
+	*ninval = n;
+	
+	return (outbuf - out);
+}
+
+
+#define USER_CHARSETS_INCLUDE_UTF8    (1 << 0)
+#define USER_CHARSETS_INCLUDE_LOCALE  (1 << 1)
+
+
+/**
+ * g_mime_utils_decode_8bit:
+ * @text: input text in unknown 8bit/multibyte character set
+ * @len: input text length
+ *
+ * Attempts to convert text in an unknown 8bit/multibyte charset into
+ * UTF-8 by finding the charset which will convert the most bytes into
+ * valid UTF-8 characters as possible. If no exact match can be found,
+ * it will choose the best match and convert invalid byte sequences
+ * into question-marks (?) in the returned string buffer.
+ *
+ * Returns a UTF-8 string representation of @text.
+ **/
+char *
+g_mime_utils_decode_8bit (const char *text, size_t len)
+{
+	const char **charsets, **user_charsets, *locale, *best;
+	size_t outleft, outlen, min, ninval;
+	unsigned int included = 0;
+	char *out, *outbuf;
+	iconv_t cd;
+	int i = 0;
+	
+	g_return_val_if_fail (text != NULL, NULL);
+	
+	locale = g_mime_locale_charset ();
+	if (locale && !g_ascii_strcasecmp (locale, "UTF-8"))
+		included |= USER_CHARSETS_INCLUDE_LOCALE;
+	
+	if ((user_charsets = g_mime_user_charsets ())) {
+		while (user_charsets[i])
+			i++;
+	}
+	
+	charsets = g_alloca (sizeof (char *) * (i + 3));
+	i = 0;
+	
+	if (user_charsets) {
+		while (user_charsets[i]) {
+			/* keep a record of whether or not the user-supplied
+			 * charsets include UTF-8 and/or the default fallback
+			 * charset so that we avoid doubling our efforts for
+			 * these 2 charsets. We could have used a hash table
+			 * to keep track of unique charsets, but we can
+			 * (hopefully) assume that user_charsets is a unique
+			 * list of charsets with no duplicates. */
+			if (!g_ascii_strcasecmp (user_charsets[i], "UTF-8"))
+				included |= USER_CHARSETS_INCLUDE_UTF8;
+			
+			if (locale && !g_ascii_strcasecmp (user_charsets[i], locale))
+				included |= USER_CHARSETS_INCLUDE_LOCALE;
+			
+			charsets[i] = user_charsets[i];
+			i++;
+		}
+	}
+	
+	if (!(included & USER_CHARSETS_INCLUDE_UTF8))
+		charsets[i++] = "UTF-8";
+	
+	if (!(included & USER_CHARSETS_INCLUDE_LOCALE))
+		charsets[i++] = locale;
+	
+	charsets[i] = NULL;
+	
 	min = len;
 	best = charsets[0];
 	
-	outlen = (len * 2) + 16; 
-	out = g_malloc (outlen + 1);
+	outleft = (len * 2) + 16;
+	out = g_malloc (outleft + 1);
 	
 	for (i = 0; charsets[i]; i++) {
 		if ((cd = g_mime_iconv_open ("UTF-8", charsets[i])) == (iconv_t) -1)
 			continue;
 		
-		outleft = outlen;
-		outbuf = out;
-		inleft = len;
-		inbuf = text;
-		n = 0;
-		
-		do {
-			rc = iconv (cd, (char **) &inbuf, &inleft, &outbuf, &outleft);
-			if (rc == (size_t) -1) {
-				if (errno == EINVAL) {
-					/* incomplete sequence at the end of the input buffer */
-					n += inleft;
-					break;
-				}
-				
-				if (errno == E2BIG) {
-					outlen += (inleft * 2) + 16;
-					rc = (size_t) (outbuf - out);
-					out = g_realloc (out, outlen + 1);
-					outbuf = out + rc;
-				} else {
-					inleft--;
-					inbuf++;
-					n++;
-				}
-			}
-		} while (inleft > 0);
-		
-		rc = iconv (cd, NULL, NULL, &outbuf, &outleft);
-		*outbuf = '\0';
+		outlen = charset_convert (cd, text, len, &out, &outleft, &ninval);
 		
 		g_mime_iconv_close (cd);
 		
-		if (rc != (size_t) -1 && n == 0)
-			return out;
+		if (ninval == 0)
+			return g_realloc (out, outlen + 1);
 		
-		if (n < min) {
+		if (ninval < min) {
 			best = charsets[i];
-			min = n;
+			min = ninval;
 		}
 	}
 	
@@ -1482,8 +1628,7 @@ decode_8bit (const char *text, size_t len)
 		 * is replace the 8bit garbage and pray */
 		register const char *inptr = text;
 		const char *inend = inptr + len;
-		
-		outbuf = out;
+		char *outbuf = out;
 		
 		while (inptr < inend) {
 			if (is_ascii (*inptr))
@@ -1492,46 +1637,19 @@ decode_8bit (const char *text, size_t len)
 				*outbuf++ = '?';
 		}
 		
-		*outbuf = '\0';
+		*outbuf++ = '\0';
 		
-		return out;
+		return g_realloc (out, outbuf - out);
 	}
 	
-	outleft = outlen;
-	outbuf = out;
-	inleft = len;
-	inbuf = text;
-	
-	do {
-		rc = iconv (cd, (char **) &inbuf, &inleft, &outbuf, &outleft);
-		if (rc == (size_t) -1) {
-			if (errno == EINVAL) {
-				/* incomplete sequence at the end of the input buffer */
-				break;
-			}
-			
-			if (errno == E2BIG) {
-				rc = outbuf - out;
-				outlen += inleft * 2 + 16;
-				out = g_realloc (out, outlen + 1);
-				outbuf = out + rc;
-			} else {
-				*outbuf++ = '?';
-				outleft--;
-				inleft--;
-				inbuf++;
-			}
-		}
-	} while (inleft > 0);
-	
-	iconv (cd, NULL, NULL, &outbuf, &outleft);
-	*outbuf = '\0';
+	outlen = charset_convert (cd, text, len, &out, &outleft, &ninval);
 	
 	g_mime_iconv_close (cd);
 	
-	return out;
+	return g_realloc (out, outlen + 1);
 }
 
+
 /* this decodes rfc2047's version of quoted-printable */
 static ssize_t
 quoted_decode (const unsigned char *in, size_t len, unsigned char *out)
@@ -1578,11 +1696,11 @@ rfc2047_decode_word (const char *in, size_t inlen)
 	const unsigned char *inend = instart + inlen - 2;
 	unsigned char *decoded;
 	const char *charset;
+	size_t len, ninval;
 	char *charenc, *p;
 	guint32 save = 0;
 	ssize_t declen;
 	int state = 0;
-	size_t len;
 	iconv_t cd;
 	char *buf;
 	
@@ -1650,19 +1768,24 @@ rfc2047_decode_word (const char *in, size_t inlen)
 			     "be corrupt: %s", charset[0] ? charset : "unspecified charset",
 			     g_strerror (errno)));
 		
-		return decode_8bit ((char *) decoded, declen);
+		return g_mime_utils_decode_8bit ((char *) decoded, declen);
 	}
 	
-	buf = g_mime_iconv_strndup (cd, (char *) decoded, declen);
+	len = declen;
+	buf = g_malloc (len + 1);
+	
+	charset_convert (cd, decoded, declen, &buf, &len, &ninval);
+	
 	g_mime_iconv_close (cd);
 	
-	if (buf != NULL)
-		return buf;
+#if w(!)0
+	if (ninval > 0) {
+		g_warning ("Failed to completely convert \"%.*s\" to UTF-8, display may be "
+			   "corrupt: %s", declen, decoded, g_strerror (errno));
+	}
+#endif
 	
-	w(g_warning ("Failed to convert \"%.*s\" to UTF-8, display may be "
-		     "corrupt: %s", declen, decoded, g_strerror (errno)));
-	
-	return decode_8bit ((char *) decoded, declen);
+	return buf;
 }
 
 
@@ -1771,7 +1894,7 @@ g_mime_utils_header_decode_text (const char *in)
 				/* append word token */
 				if (!ascii) {
 					/* *sigh* I hate broken mailers... */
-					decoded = decode_8bit (text, n);
+					decoded = g_mime_utils_decode_8bit (text, n);
 					g_string_append (out, decoded);
 					g_free (decoded);
 				} else {
@@ -1870,7 +1993,7 @@ g_mime_utils_header_decode_phrase (const char *in)
 			
 			if (!ascii) {
 				/* *sigh* I hate broken mailers... */
-				decoded = decode_8bit (text, n);
+				decoded = g_mime_utils_decode_8bit (text, n);
 				g_string_append (out, decoded);
 				g_free (decoded);
 			} else {
@@ -2071,8 +2194,10 @@ rfc2047_encode_get_rfc822_words (const char *in, gboolean phrase)
 				tail = word;
 				count = 0;
 				
+				/* Note: don't reset 'type' as it
+				 * needs to be preserved when breaking
+				 * long words */
 				start = inptr;
-				type = WORD_ATOM;
 				encoding = 0;
 			}
 		}
@@ -2092,6 +2217,18 @@ rfc2047_encode_get_rfc822_words (const char *in, gboolean phrase)
 		tail = word;
 	}
 	
+#if d(!)0
+	printf ("rfc822 word tokens:\n");
+	word = words;
+	while (word) {
+		printf ("\t'%.*s'; type=%d, encoding=%d\n",
+			word->end - word->start, word->start,
+			word->type, word->encoding);
+		
+		word = word->next;
+	}
+#endif
+	
 	return words;
 }
 
@@ -2268,9 +2405,14 @@ rfc2047_encode (const char *in, gushort safemask)
 				start = word->start;
 			}
 			
-			if (word->encoding == 1) {
+			switch (word->encoding) {
+			case 0: /* us-ascii */
+				rfc2047_encode_word (out, start, len, "us-ascii", safemask);
+				break;
+			case 1: /* iso-8859-1 */
 				rfc2047_encode_word (out, start, len, "iso-8859-1", safemask);
-			} else {
+				break;
+			default:
 				charset = NULL;
 				g_mime_charset_init (&mask);
 				g_mime_charset_step (&mask, start, len);
@@ -2286,7 +2428,9 @@ rfc2047_encode (const char *in, gushort safemask)
 					charset = g_mime_charset_best_name (&mask);
 				
 				rfc2047_encode_word (out, start, len, charset, safemask);
+				break;
 			}
+			
 			break;
 		}
 		
